@@ -114,29 +114,37 @@ let parse args =
 
 let () = 
   
-  let pos = ref 0 in
-  while !pos < Array.length Sys.argv && Sys.argv.(!pos) <> 
-    "-read_config_file"
-  do incr pos done;
+  try 
+    let pos = ref 0 in
+    while !pos < Array.length Sys.argv && Sys.argv.(!pos) <> 
+      "-read_config_file"
+    do incr pos done;
 
-  if !pos = Array.length Sys.argv 
-  then (
-    parse Sys.argv;
-    let from_file_commandline = 
-      fname_convert (Filename.concat !basedir config_fname)
-    in
-    parse from_file_commandline
-  )
-  else (
-    parse (Sys.argv <|> (0,!pos));
-    let from_file_commandline = 
-      fname_convert (Filename.concat !basedir config_fname)
-    in
-    parse from_file_commandline;
-    parse (Array.append [|""|] (Sys.argv <|> (!pos + 1,0)))
-  );
+    if !pos = Array.length Sys.argv 
+    then (
+      parse Sys.argv;
+      let from_file_commandline = 
+	fname_convert (Filename.concat !basedir config_fname)
+      in
+      parse from_file_commandline
+    )
+    else (
+      parse (Sys.argv <|> (0,!pos));
+      let from_file_commandline = 
+	fname_convert (Filename.concat !basedir config_fname)
+      in
+      parse from_file_commandline;
+      parse (Array.append [|""|] (Sys.argv <|> (!pos + 1,0)))
+    );
 
-  anonlist := List.rev !anonlist;
-  prepare_options ()
+    anonlist := List.rev !anonlist;
+    prepare_options ()
 
+  with
+    | Arg.Bad s ->
+	print_string s;
+	exit (-1)
+    | Arg.Help s -> 
+	print_string s; 
+	exit 0
   
