@@ -32,6 +32,7 @@ module Map = PMap.Map
 type siginfo = { mutable userid: string option;
 		 mutable policy_url: string option;
 		 mutable notation_data: (string * string) option;
+		 mutable revocation_key: string option;
 		 mutable is_primary_uid: bool;
 		 mutable keyid: string option;
 		 mutable sigtype: int;
@@ -46,6 +47,7 @@ let empty_siginfo () =
   { userid = None;
     policy_url = None;
     notation_data = None;
+    revocation_key = None;
     is_primary_uid = false;
     keyid = None;
     sigtype = 0;
@@ -73,61 +75,70 @@ let sig_to_siginfo sign =
 	  siginfo.keyid <- Some s.v3s_keyid;
 	  siginfo.sig_creation_time <- Some s.v3s_ctime
       | V4sig s ->
+	  let update_siginfo ssp = 
+	    match ssp.ssp_type with
+
+	      | 2 -> (* sign. expiration time *)
+		  if ssp.ssp_length = 4 then
+		    siginfo.sig_creation_time <-
+		    Some (ParsePGP.int64_of_string ssp.ssp_body)
+
+	      | 3 -> (* sign. expiration time *)
+		  if ssp.ssp_length = 4 then
+		    siginfo.sig_expiration_time <-
+		    let exp = ParsePGP.int64_of_string ssp.ssp_body in
+		    if Int64.compare exp Int64.zero = 0 
+		    then None else Some exp
+
+	      | 9 -> (* key expiration time *)
+		  if ssp.ssp_length = 4 then
+		    siginfo.key_expiration_time <-
+		    let exp = ParsePGP.int64_of_string ssp.ssp_body in
+		    if Int64.compare exp Int64.zero = 0 
+		    then None else Some exp
+
+	      | 12 -> (* revocation key *)
+		  let cin = new Channel.string_in_channel ssp.ssp_body 0 in
+		  let revclass = cin#read_int_size 1 in
+		  let algid = cin#read_int_size 1 in
+		  let fingerprint = cin#read_string 20 in
+		  siginfo.revocation_key <- Some fingerprint
+
+	      | 16 -> (* issuer keyid *)
+		  if ssp.ssp_length = 8 then
+		    siginfo.keyid <- Some ssp.ssp_body 
+		  else
+		    printf "Argh!  that makes no sense: %d\n" ssp.ssp_length 
+
+	      | 20 -> (* notation data *)
+		  let cin = new Channel.string_in_channel ssp.ssp_body 0 in
+		  let flags = cin#read_string 4 in
+		  let name_len = cin#read_int_size 2 in
+		  let value_len = cin#read_int_size 2 in
+		  let name_data = cin#read_string name_len in
+		  let value_data = cin#read_string value_len in
+
+		  if Char.code flags.[0] = 0x80 then 
+		    (* human-readable notation data *)
+		    siginfo.notation_data <- Some (name_data,value_data)
+
+	      | 25 -> (* primary userid (bool) *)
+		  if ssp.ssp_length = 1 then
+		    let v = int_of_char ssp.ssp_body.[0] in
+		    siginfo.is_primary_uid <- v <> 0
+
+	      | 26 -> (* policy URL *)
+		  siginfo.policy_url <- Some ssp.ssp_body
+
+	      | 28 -> (* signer's userid *)
+		  siginfo.userid <- Some ssp.ssp_body
+
+	      | _ -> (* miscellaneous other packet *)
+		  ()
+	  in
 	  siginfo.sigtype <- s.v4s_sigtype;
 	  List.iter (s.v4s_hashed_subpackets @ s.v4s_unhashed_subpackets)
-	    ~f:(fun ssp -> match ssp.ssp_type with
-
-		  | 2 -> (* sign. expiration time *)
-		      if ssp.ssp_length = 4 then
-			siginfo.sig_creation_time <-
-			Some (ParsePGP.int64_of_string ssp.ssp_body)
-
-		  | 3 -> (* sign. expiration time *)
-		      if ssp.ssp_length = 4 then
-			siginfo.sig_expiration_time <-
-			let exp = ParsePGP.int64_of_string ssp.ssp_body in
-			if Int64.compare exp Int64.zero = 0 
-			then None else Some exp
-
-		  | 9 -> (* key expiration time *)
-		      if ssp.ssp_length = 4 then
-			siginfo.key_expiration_time <-
-			let exp = ParsePGP.int64_of_string ssp.ssp_body in
-			if Int64.compare exp Int64.zero = 0 
-			then None else Some exp
-
-		  | 16 -> (* issuer keyid *)
-		      if ssp.ssp_length = 8 then
-			siginfo.keyid <- Some ssp.ssp_body 
-		      else
-			printf "Argh!  that makes no sense: %d\n" ssp.ssp_length 
-
-		  | 20 -> (* notation data *)
-		      let cin = new Channel.string_in_channel ssp.ssp_body 0 in
-		      let flags = cin#read_string 4 in
-		      let name_len = cin#read_int_size 2 in
-		      let value_len = cin#read_int_size 2 in
-		      let name_data = cin#read_string name_len in
-		      let value_data = cin#read_string value_len in
-
-		      if Char.code flags.[0] = 0x80 then 
-			(* human-readable notation data *)
-			siginfo.notation_data <- Some (name_data,value_data)
-
-		  | 25 -> (* primary userid (bool) *)
-		      if ssp.ssp_length = 1 then
-			let v = int_of_char ssp.ssp_body.[0] in
-			siginfo.is_primary_uid <- v <> 0
-
-		  | 26 -> (* policy URL *)
-		      siginfo.policy_url <- Some ssp.ssp_body
-
-		  | 28 -> (* signer's userid *)
-		      siginfo.userid <- Some ssp.ssp_body
-
-		  | _ -> (* miscellaneous other packet *)
-		      ()
-	       )
+	    ~f:(fun ssp -> try update_siginfo ssp with End_of_file -> ())
   end;
   siginfo
 
@@ -254,12 +265,23 @@ let siginfo_to_lines ~get_uid ?key_creation_time request self_keyid siginfo =
   let notation_data_opt = 
     apply_opt siginfo.notation_data
       ~f:(fun (name,value) ->
-	    sprintf "    Notation data: <u>%s</u> %s"
+  	    sprintf "    Notation data: <u>%s</u> %s"
 	    (HtmlTemplates.html_quote name)
 	    (HtmlTemplates.html_quote value)
 	 )
   in
-  firstline :: filter_opts [policy_url_opt; notation_data_opt]
+  let revocation_key_opt = 
+    apply_opt siginfo.revocation_key 
+      ~f:(fun fingerprint ->
+	    sprintf "    Revocation key fingerprint: <a href=\"%s\">%s</a>"
+	    (HtmlTemplates.link ~hash:request.hash ~op:"vindex" ~hostname:!Settings.hostname
+	       ~port:http_port ~fingerprint:request.fingerprint 
+	       ~keyid:(Utils.hexstring fingerprint)
+	    )
+	    (Fingerprint.fp_to_string fingerprint)
+	 )
+  in
+  firstline :: filter_opts [policy_url_opt; notation_data_opt; revocation_key_opt]
 
 
 (********************************************************************)
