@@ -174,10 +174,6 @@ struct
     let dbs = get_dbs () in
     dbs.dump.filearray
 
-  let get_num_keys () = 
-    let dbs = get_dbs () in
-    Db.get_size dbs.key 
-
   (***********************************************************************)
   (*  Key conversions ****************************************************)
   (***********************************************************************)
@@ -322,7 +318,7 @@ struct
 	      | Some cache_bytes -> Dbenv.set_cachesize dbenv 
 		  ~gbytes:0 ~bytes:cache_bytes ~ncache:0);
 	  Dbenv.dopen dbenv settings.dbdir 
-	    ( [ Dbenv.INIT_MPOOL; Dbenv.CREATE; (*Dbenv.INIT_LOCK*) ]
+	    ( [ Dbenv.INIT_MPOOL; Dbenv.CREATE; (* Dbenv.INIT_LOCK *) ]
 	      @ ( if settings.withtxn then [ Dbenv.INIT_TXN; Dbenv.RECOVER ]
 		  else [] ) )
 	    0o600;
@@ -618,6 +614,22 @@ struct
       
   (***********************************************************************)
 
+  let keyid_iter ~f =
+    let dbs = get_dbs () in
+    let c = Cursor.create dbs.keyid in
+    let rec loop get_type =
+      match (try Some (Cursor.get c get_type []) with Not_found -> None)
+      with
+	| Some (key,data) -> 
+	    f ~keyid:key ~hash:data; 
+	    loop Cursor.NEXT 
+	| None -> ()
+    in
+    protect ~f:(fun () -> loop Cursor.FIRST)
+      ~finally:(fun () -> Cursor.close c)
+
+  (***********************************************************************)
+
   let raw_iter ~f =
     let dbs = get_dbs () in
     let c = Cursor.create dbs.key in
@@ -1308,6 +1320,7 @@ struct
 
   (**********************************************************)
 
+(*
   (** atomically remove all keys on [delete_list] and add key [newkey] *)
   let replace delete_list newkey =
     let txn = txn_begin () in
@@ -1322,6 +1335,30 @@ struct
       | e -> 
 	  txn_abort txn;
 	  raise e
+*)
+
+  (** atomically remove all keys on [delete_list] and add key [newkey] *)
+  let replace delete_list newkey = 
+    let txn = txn_begin () in
+    try 
+      let newkey_update = (key_to_metadata newkey, AddKey) in
+      let delete_updates = 
+	List.map ~f:(fun key -> (key_to_metadata key,DeleteKey)) delete_list in
+      apply_md_updates_txn ~txn (Array.of_list (sort_remove (newkey_update::delete_updates)));
+      txn_commit txn
+    with
+      | Bdb.DBError s as e -> 
+	  signore (eplerror 0 e "Fatal database error");
+	  raise Sys.Break
+      | e -> 
+	  txn_abort txn;
+	  raise e
+
+
+  let get_num_keys () = 
+    let ctr = ref 0 in
+    keyid_iter ~f:(fun ~keyid ~hash -> incr ctr);
+    !ctr
 
 end
 
